@@ -15,7 +15,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: bayes.c,v 1.10 2002/08/12 21:13:51 beerli Exp $
+ $Id: bayes.c,v 1.14 2002/11/27 16:17:13 beerli Exp $
 */
 
 #include "bayes.h"
@@ -77,7 +77,7 @@ double probWait(vtlist *tlp, world_fmt *world, long numpop)
 boolean
 bayes_accept (double newval, double oldval, double heat)
 {
-	double diff = (newval - oldval) * heat;
+	double diff = (newval - oldval) * heat;// heat = 1/temperature
     if (diff > 0.0)
         return TRUE;
     if (log (RANDUM ()) < diff)
@@ -91,13 +91,20 @@ double
 propose_newparam (double param,
                   double delta, double minparam, double maxparam)
 {
+    double np;
     double r = RANDUM ();
-    double sign = RANDUM () < 0.5 ? -1. : 1.;
-    double np = param + sign * r * delta;
-    if (np < minparam)
-        return minparam - np;
-    if (np > maxparam)
-        return np - maxparam;
+    if(r>0.5)
+      {
+        np = param + (2. * r - 1.) * delta;
+        if (np > maxparam)
+            return 2. * maxparam - np;
+      }
+    else
+      {
+        np = param - 2. * r * delta;
+        if (np < minparam)
+            return 2. * minparam - np;
+      }        
     return np;
 }
 
@@ -105,13 +112,11 @@ long
 bayes_update (world_fmt * world)
 {
     long ba=0;
-//    long j;
     long i = world->bayes->paramnum;
     double oldparam;
 	double oldval;
     double newval;
     double newparam;
-	 double heat=0.;
 	if(((1+world->bayes->count++) % ((long)world->options->updateratio)) != 0)
 		return 0;
     if(!strchr("c0",world->options->custm2[world->bayes->paramnum]))
@@ -124,20 +129,16 @@ bayes_update (world_fmt * world)
         oldparam = world->param0[i];
         world->param0[i]=newparam;
         newval =probg_treetimes(world);
-		if(world->options->heating)
-			heat = world->heat[0];
-		else
-			heat = 1;
-        if(bayes_accept(newval, oldval,heat))
+        if(bayes_accept(newval, oldval,world->heat))
         {
-//			printf ("***%f (%f) %f %f %f %li\n",newparam, oldparam, newval,oldval, world->likelihood[world->G], i);
+		//	printf ("***%f (%f) %f %f %f %li\n",newparam, oldparam, newval,oldval, world->likelihood[world->G], i);
             world->bayes->oldval = newval;
             precalc_world(world);
             ba = 1;
         }
         else
         {
-//			printf ("   %f (%f) %f %f %f %li\n",newparam, oldparam, newval,oldval, world->likelihood[world->G], i);
+       // printf ("   %f (%f) %f %f %f %li\n",newparam, oldparam, newval,oldval, world->likelihood[world->G], i);
             world->param0[i] = oldparam;
             world->bayes->oldval = oldval;
             ba = 0;
@@ -157,11 +158,17 @@ void bayes_save(world_fmt *world)
     world->bayes->params[pnum][0] = world->bayes->oldval + //probg_treetimes(world) +
 		world->likelihood[world->G];
     memcpy(&world->bayes->params[pnum][1], world->param0,sizeof(double)*world->numpop2);
-    //DEBUG
-    //for(i=0; i<world->numpop2+1;i++)
-    //    FPRINTF(stderr,"%f ",world->bayes->params[pnum][i]);
-    //FPRINTF(stderr,"\n");
-
+    if(world->options->verbose)
+      {
+        FPRINTF(stdout,"%8li> %10.2f ",world->bayes->count, world->bayes->params[pnum][0]);
+        
+        FPRINTF(stdout,"%8li> %10.2f ",world->bayes->count, world->bayes->params[pnum][0]);
+        for(i=1; i<world->numpop + 1;i++)
+        FPRINTF(stdout,"%6.4f ",world->bayes->params[pnum][i]);
+        for(i= world->numpop + 1; i<world->numpop2+1;i++)
+        FPRINTF(stdout,"%6.1f ",world->bayes->params[pnum][i]);
+        FPRINTF(stdout,"\n");
+      }
     pnum++;
     if(pnum>=allocparams)
     {
@@ -192,13 +199,13 @@ void bayes_fill(world_fmt *world, option_fmt *options)
     long i;
     for(i=0; i< world->numpop;i++)
     {
-        world->bayes->delta[i] = 0.001;
+        world->bayes->delta[i] = 0.01;
         world->bayes->minparam[i] = SMALLEST_THETA;
         world->bayes->maxparam[i] = BIGGEST_THETA;
     }
     for(i=world->numpop; i< world->numpop2;i++)
     {
-        world->bayes->delta[i] = 10;//data dependent and certainly bad for ep data
+        world->bayes->delta[i] = 10.;
         world->bayes->minparam[i] = SMALLEST_MIGRATION;
         world->bayes->maxparam[i] = BIGGEST_MIGRATION;
     }
@@ -221,77 +228,96 @@ void bayes_free(world_fmt *world)
 
 void bayes_stat(world_fmt *world)
 {
-    long i, j;
-    bayes_fmt *bayes = world->bayes;
-	long size;
-	long numpop2 = world->numpop2;
-	long mode;
-	long weight;
+  long frompop=0;
+  long topop=0;
+  long i, j;
+  bayes_fmt *bayes = world->bayes;
+  long size;
+  long numpop2 = world->numpop2;
+  long mode;
+  long weight;
 #ifdef LONGSUM
-	long addon = (world->fluctuate ? world->numpop * 3 : 0);
+  long addon = (world->fluctuate ? world->numpop * 3 : 0);
 #else
-	long addon = 0;
+  long addon = 0;
 #endif
-	double temp;
-	double **rawstat;
-	double *averages, *waverages;
-	double *std, *wstd;
-	addon += world->options->gamma ? 1 : 0;
-	size = numpop2 + addon;
-    averages = calloc(size,sizeof(double));
-    std = calloc(size,sizeof(double));
-    waverages = calloc(size,sizeof(double));
-    wstd = calloc(size,sizeof(double));
-    rawstat = calloc(size,sizeof(double *));
-    rawstat[0] = calloc(size * bayes->numparams,sizeof(double));
-
-    for(j=1; j< size; j++)
-        rawstat[j] = rawstat[0] + j * bayes->numparams;
-	mode = 0;
-	// find the mode so that we can (1) have the mode and (2) know the max weight for the
-	// weighted average.
-    for(i=0; i< bayes->numparams; i++)
-	  {
-		mode =  (bayes->params[i][0] > bayes->params[mode][0] ? i : mode);
-	  }
-	for(i=0; i< bayes->numparams; i++)
+  double temp;
+  double **rawstat;
+  double *averages;
+  double *std;
+  char stemp[9];
+  addon += world->options->gamma ? 1 : 0;
+  size = numpop2 + addon;
+  averages = calloc(size,sizeof(double));
+  std = calloc(size,sizeof(double));
+//  waverages = calloc(size,sizeof(double));
+//  wstd = calloc(size,sizeof(double));
+  rawstat = calloc(size,sizeof(double *));
+  rawstat[0] = calloc(size * bayes->numparams,sizeof(double));
+  
+  for(j=1; j< size; j++)
+    rawstat[j] = rawstat[0] + j * bayes->numparams;
+  mode = 0;
+  // find the mode so that we can (1) have the mode and (2) know the max weight for the
+  // weighted average.
+  for(i=0; i< bayes->numparams; i++)
     {
-		FPRINTF(world->bayesfile, "%f ", bayes->params[i][0]);
-		weight =  exp(bayes->params[i][0] -  bayes->params[mode][0]);
-        for(j = 0; j < size; j++)
-        {
-			   temp = bayes->params[i][j+1];
-            FPRINTF(world->bayesfile, "%f ", temp);
-            waverages[j] += weight*temp;
-            wstd[j] += weight * temp * temp;
-            averages[j] += temp;
-            std[j] += temp * temp;
-            rawstat[j][i] = temp;
-        }
-        FPRINTF(world->bayesfile, "\n");
+      mode =  (bayes->params[i][0] > bayes->params[mode][0] ? i : mode);
     }
-	FPRINTF(world->outfile,"\n\n\nBayesian estimates\n");
-	FPRINTF(world->outfile,"==================\n\n");
-	FPRINTF(world->outfile,"Parameter 2.5%%  median 97.5%%  mode   mean  std  wmean   wstd\n");
-	FPRINTF(world->outfile,"------------------------------------------------------------------------------\n");
-    for(j=0; j< size; j++)
+  for(i=0; i< bayes->numparams; i++)
     {
-        qsort(rawstat[j],bayes->numparams,sizeof(double),numcmp);
-        FPRINTF(world->outfile,"  %3li   ",j+1);
-		FPRINTF(world->outfile, " %.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f\n",
+      FPRINTF(world->bayesfile, "%f ", bayes->params[i][0]);
+      weight =  exp(bayes->params[i][0] -  bayes->params[mode][0]);
+      for(j = 0; j < size; j++)
+        {
+	  temp = bayes->params[i][j+1];
+	  FPRINTF(world->bayesfile, "%f ", temp);
+	  averages[j] += temp;
+	  std[j] += temp * temp;
+	  rawstat[j][i] = temp;
+        }
+      FPRINTF(world->bayesfile, "\n");
+    }
+  FPRINTF(world->outfile,"\n\n\nBayesian estimates\n");
+  FPRINTF(world->outfile,"==================\n\n");
+  FPRINTF(world->outfile,"Parameter  2.5%%     median    97.5%%     mode     mean      std\n");
+  FPRINTF(world->outfile,"--------------------------------------------------------------\n");
+  for(j=0; j< size; j++)
+    {
+      qsort(rawstat[j],bayes->numparams,sizeof(double),numcmp);
+      if(j < world->numpop)
+	{
+	  FPRINTF(world->outfile,"Theta_%-3li",j+1);
+	  FPRINTF(world->outfile, "%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
 		  rawstat[j][(long)(bayes->numparams * 0.025)],
 		  rawstat[j][(long)(bayes->numparams * 0.5)],
 		  rawstat[j][(long)(bayes->numparams * 0.975)],
 		  rawstat[j][mode],
 		  averages[j]/bayes->numparams,
-		  std[j]/(bayes->numparams-1), waverages[j], wstd[j]);
+		  sqrt(std[j]/(bayes->numparams-1)));
+	}
+      else
+	{
+	  topop = (long) (j-world->numpop)/(world->numpop-1);
+	  frompop = j - world->numpop - topop * (world->numpop - 1);
+	  if(frompop>=topop)
+	    frompop += 1;
+	  sprintf(stemp,"M_%li->%li", frompop+1, topop+1);
+	  FPRINTF(world->outfile, "%-9.9s%8.1f %8.1f %8.1f %8.1f %8.1f %8.1f\n",
+		  stemp, rawstat[j][(long)(bayes->numparams * 0.025)],
+		  rawstat[j][(long)(bayes->numparams * 0.5)],
+		  rawstat[j][(long)(bayes->numparams * 0.975)],
+		  rawstat[j][mode],
+		  averages[j]/bayes->numparams,
+		  std[j]/(bayes->numparams-1));
+	}     
     }
-    free(rawstat[0]);
-    free(rawstat);
-    free(averages);
-    free(std);
-    free(waverages);
-    free(wstd);
+  free(rawstat[0]);
+  free(rawstat);
+  free(averages);
+  free(std);
+ // free(waverages);
+ // free(wstd);
 }
 
 
