@@ -18,7 +18,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
- $Id: broyden.c,v 1.65 2002/08/12 21:13:52 beerli Exp $
+ $Id: broyden.c,v 1.66 2002/12/22 04:50:39 beerli Exp $
 -------------------------------------------------------*/
 extern double norm (double *d, long size);
 
@@ -55,7 +55,7 @@ double probG2 (double *param, double *lparam, double *sm, double *kt,
                long numpop);
 void probG3 (double *apg, double *apg0r, timearchive_fmt * tyme, long numpop,
              double *apgmax, double *param, double *lparam, double *sm);
-double probG4 (double *fullparam, double *data, long numpop, long numpop2);
+inline double probG4 (double *fullparam, double *data, long numpop, long numpop2);
 double logprob_noevent (world_fmt * world, long interval);
 double sum_migprob (world_fmt * world, long pop, long interval);
 double sum_mig (double *param, long msta, long msto);
@@ -950,11 +950,11 @@ param_all_adjust (double *xv, nr_fmt *nr)
         consts = wopt->constparam;
     p = custm;
     z = strcspn(p, "m");
-    if (z > 0 && z < numpop)
+    if (z < numpop)
         force_sametheta (param, wopt, numpop);
     p = custm + numpop;
     z = strcspn(p, "m");
-    if (z > 0 && z < numpop*(numpop-1))
+    if (z < numpop*(numpop-1))
         force_samemigration (param, wopt, numpop);
     for (i = 0; i < zeron; i++)
         param[zeros[i]] = -30.; //-DBL_MAX;
@@ -1714,7 +1714,7 @@ probG (double *param, double *lparam, tarchive_fmt * tl, nr_fmt * nr,
     return result;
 }
 
-double
+inline double
 probG4 (double *fullparam, double *data, long numpop, long numpop2)
 {
     int i;
