@@ -23,7 +23,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: combroyden2.c,v 1.82 2002/08/12 21:13:52 beerli Exp $
+$Id: combroyden2.c,v 1.84 2002/12/22 04:50:40 beerli Exp $
  
 -------------------------------------------------------*/
 
@@ -79,6 +79,9 @@ void print_menu_finalestimate (boolean progress, long locus, long loci);
 
 /* start parameter for multilocus estimator, creates simple average */
 void calc_meanparam (world_fmt * world, long n, long repstart, long repstop);
+
+void expected_param(double *param, world_fmt *world);
+
 /* multilocus first derivatives with or without gamma distrib. mut. rate */
 void combine_gradient (nr_fmt * nr, helper_fmt * helper, double *gxv);
 #ifdef SLOWNET
@@ -171,6 +174,7 @@ void finish_broyden (double newL, double normd, long trials,
                      long analystype, long repkind);
 void fill_profile_index (nr_fmt * nr);
 
+
 /* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
 /* public functions---------------------------------------------- */
 /* driver for maximizing single locus, multilocus functions      */
@@ -210,21 +214,23 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
     doublevec1d (&param, nr->partsize + 1);
 
 #ifdef MPI
-
     if (myID == MASTER)
-    {
-        // just use last world->param0 this might need improvement
-        // and perhaps even gathering of param0 from slave nodes
-    }
+        memcpy (param, world->param0, sizeof (double) * nr->numpop2);
 #else
     if (kind == MULTILOCUS || repkind != SINGLECHAIN)
     {
         set_replicates (world, world->repkind, world->rep, &repstart, &repstop);
         calc_meanparam (world, world->numpop2, repstart, repstop);
+        memcpy (param, world->param0, sizeof (double) * nr->numpop2);
     }
+//    else
+  //    {
+    //    expected_param(param,world);
+     // }
 #endif
 
     memcpy (param, world->param0, sizeof (double) * nr->numpop2);
+
 #ifdef LONGSUM
 
     memcpy (param+nr->numpop2+
@@ -251,7 +257,7 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
         if (world->options->logfile)
         {
             FPRINTF (world->options->logfile,
-                     "           @Maximization cycles needed: %li\n           Norm of first derivatives:  %f\n", trials, nr->normd);
+                     "           Maximization cycles needed: %li\n           Norm of first derivatives:  %f\n", trials, nr->normd);
         }
     }
     if (kind == MULTILOCUS && world->options->plotnow && world->loci > 1)
@@ -1019,6 +1025,55 @@ quadratic_lamda (double lamda, double a, double b, double c, double la,
 
 }
 
+#define TAU1  2.6180339887
+#define TAU   1.6180339887
+#define BSTART  -2.360679775
+double golden_section(double (*fpsi) (double lamda, helper_fmt * helper, double *param,
+    double *lparam), helper_fmt * thishelper,
+    double *tempparam, double *templparam)
+{
+    double a = -10.;
+    double b = BSTART;
+    double c = 10.;
+    double bl;
+    double d=1.;
+    double dl = -DBL_MAX;
+    double ll = 0.;
+    helper_fmt helper;
+    helper = *thishelper;
+    bl = (*fpsi) (b, &helper, tempparam, templparam);
+    while(fabs(ll - dl)>EPSILON)
+      {
+        ll = dl;
+        if((c-b) > (b-a))
+          {
+            d = b + (c - b)/ TAU1;
+            dl = (*fpsi) (d, &helper, tempparam, templparam);
+            if(dl > bl)
+                c  = d;
+            else
+              {
+                a  = b;
+                b  = d;
+                bl = dl;
+              }
+          }            
+        else
+          {
+            d = a + (b - a)/ TAU1;
+            dl = (*fpsi) (d, &helper, tempparam, templparam);
+            if(dl > bl)
+                a = d;
+            else
+              {
+                c = b;
+                b = d;
+                bl = dl;
+              }
+          }        
+      }
+    return d;
+}
 
 long
 do_profiles (world_fmt * world, nr_fmt * nr, double *likes,
@@ -1425,11 +1480,13 @@ maximize (double *param, world_fmt * world, nr_fmt * nr, long analystype,
 #endif
 
         newL = -DBL_MAX;
-        if (helper.boolgamma)
-            lamda = line_search (dv, gxv, psilo, &helper, oldL, tempparam, templparam);
-        else
-            lamda =     line_search (dv, gxv, psilo, &helper, oldL, tempparam, templparam);
-        helper.lamda = lamda;
+   //     if (helper.boolgamma)
+    //        lamda = golden_section (psilo, &helper, tempparam, templparam);
+        //line_search (dv, gxv, psilo, &helper, oldL, tempparam, templparam);
+     //   else
+     //       lamda = golden_section (psilo, &helper, tempparam, templparam);
+                //line_search (dv, gxv, psilo, &helper, oldL, tempparam, templparam);
+        helper.lamda = lamda = 1.;
         while ((newL <= oldL || isnan (newL) || newL <= -DBL_MAX) &&
                 fabs (lamda) > 10. * DBL_EPSILON)
         {
@@ -1654,3 +1711,24 @@ fill_helper (helper_fmt * helper, double *param, double *lparam,
     helper->atl = world->atl;
 }
 
+void expected_param(double *param, world_fmt *world)
+{
+    long i, j, tree;
+    tarchive_fmt *thistree;
+    long T = world->atl[world->replicate][world->locus].T;
+    memset(param,0,sizeof(double) * world->numpop2);
+    for(tree=0; tree < T; tree++)
+      {
+        thistree = &world->atl[world->replicate][world->locus].tl[tree];
+        for(i=0;i<world->numpop; i++)
+          {
+            param[i] += thistree->kt[i] / 2.;
+            for(j=world->mstart[i];i<world->mend[i]; i++)
+                param[i] += 1./thistree->km[i];
+          }
+      }
+    for(i=0;i<world->numpop2; i++)
+      {
+        param[i] /= T;
+      }
+}
