@@ -21,7 +21,7 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c,v 1.43 2002/08/12 22:09:49 beerli Exp $
+$Id: data.c,v 1.44 2002/12/22 04:50:42 beerli Exp $
  
 -------------------------------------------------------*/
 #include <string.h>
@@ -846,9 +846,12 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     long locus;
     long pop;
     long numind;
+    long nummiss;
     char dstring[LINESIZE];
     long *total;
+    long *totalmiss;
     total = calloc(data->loci,sizeof(long));
+    totalmiss = calloc(data->loci,sizeof(long));
     fprintf (file, "Summary of data:\n");
     fprintf (file, "---------------\n");
     switch (options->datatype)
@@ -876,37 +879,72 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     fprintf (file, "Number of loci:                          %20li\n\n",
              data->loci);
 
+    if (!strchr (SEQUENCETYPES, options->datatype))
+      {
+        fprintf (file,
+                 "Population                                    Locus  Gene copies\n");
+        fprintf (file,
+             "                                                     data (missing)\n");
+    }
+    else
+        fprintf (file,
+             "Population                                    Locus  Gene copies\n");
     fprintf (file,
-             "Population                                     Locus    Gene copies\n");
-    fprintf (file,
-             "-------------------------------------------------------------------\n");
+             "----------------------------------------------------------------\n");
     for (pop = 0; pop < world->numpop; pop++)
     {
         if (!strchr (SEQUENCETYPES, options->datatype))
-            numind = data->numalleles[pop][0] - find_missing(data,pop,0);
+          {
+            nummiss = find_missing(data,pop,0);
+            numind = data->numalleles[pop][0] - nummiss;
+            fprintf (file, "%3li %-40.40s %5li %6li (%li)\n", pop+1, data->popnames[pop], 1L , numind, nummiss);
+          }
         else
+          {
+            nummiss = 0;
             numind = data->numind[pop][0];
-        fprintf (file, "%3li %-40.40s %5li %6li\n", pop+1, data->popnames[pop], 1L , numind);
+            fprintf (file, "%3li %-40.40s %5li %6li\n", pop+1, data->popnames[pop], 1L , numind);
+          }
         total[0] += numind;
+        totalmiss[0] += nummiss;
+        
         for(locus=1; locus< data->loci; locus++)
         {
             if (!strchr (SEQUENCETYPES, options->datatype))
-                numind = data->numalleles[pop][locus] - find_missing(data,pop,locus);
+              {
+                nummiss = find_missing(data,pop,locus);
+                numind = data->numalleles[pop][locus] - nummiss;
+                fprintf (file,"                                             %5li %6li (%li)\n",  locus+1, numind, nummiss);
+              }
             else
+              {
+                nummiss=0;
                 numind = data->numind[pop][locus];
-            fprintf (file,"                                             %5li %6li\n",  locus+1, numind);
+                fprintf (file,"                                             %5li %6li\n",  locus+1, numind);
+              }
             total[locus] += numind;
+            totalmiss[locus] += nummiss;
         }
     }
-    fprintf (file,
-             "Total of all populations                     %5li %6li\n",1L, total[0]);
-    for(locus=1; locus< data->loci; locus++)
-    {
-        fprintf (file,
-                 "                                             %5li %6li\n",locus+1, total[locus]);
-    }
+    if (!strchr (SEQUENCETYPES, options->datatype))
+      {
+        fprintf (file,"Total of all populations                     %5li %6li (%li)\n",1L, total[0], totalmiss[0]);
+        for(locus=1; locus< data->loci; locus++)
+          {
+            fprintf (file,"                                             %5li %6li (%li)\n",locus+1, total[locus], totalmiss[locus]);
+          }
+      }
+    else
+      {
+        fprintf (file,"Total of all populations                     %5li %6li\n",1L, total[0]);
+        for(locus=1; locus< data->loci; locus++)
+          {
+            fprintf (file,"                                             %5li %6li\n",locus+1, total[locus]);
+          }
+      }    
     fprintf(file,"\n");
     free(total);
+    free(totalmiss);
     fflush (file);
 }
 
