@@ -21,7 +21,7 @@ and of course the documentation at:
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: main.c,v 1.85 2002/08/12 21:13:53 beerli Exp $
+$Id: main.c,v 1.91 2002/12/22 04:50:45 beerli Exp $
 -------------------------------------------------------*/
 #include "migration.h"
 #include "definitions.h"
@@ -94,7 +94,7 @@ setup_locus(long locus, world_fmt * world, option_fmt * options,
 
 void
 condense_time(world_fmt * world, long *step, long *j,
-              long *accepted, long *G, long *steps, long oldsteps,
+              double *accepted, long *G, long *steps, long oldsteps,
               long rep);
 
 long            set_repkind(option_fmt * options);
@@ -117,15 +117,23 @@ void
 combine_loci(char type, option_fmt * options, world_fmt * world,
              long *Gmax);
 
+void set_penalizer(long chain, long chains, char type, world_fmt **universe);
+
 void
 run_sampler(option_fmt * options, data_fmt * data,
             world_fmt ** universe, int usize, long *outfilepos,
             long *Gmax);
 
-void
-run_loci(world_fmt ** universe, int usize, option_fmt * options,
-         data_fmt * data, tpool_t * heating_pool, long maxreplicate,
-         long *treefilepos, long *Gmax);
+void run_replicate(long locus,
+                   long replicate,
+                   world_fmt **universe,
+                   option_fmt *options,
+                   data_fmt *data,	
+		   tpool_t * heating_pool,
+		   int usize,  
+		   long *treefilepos, 
+		   long *Gmax);
+
 
 void
 run_locus(world_fmt ** universe, int usize, option_fmt * options,
@@ -133,10 +141,12 @@ run_locus(world_fmt ** universe, int usize, option_fmt * options,
           long locus, long *treefilepos, long *Gmax);
 
 void
-heated_swap(world_fmt ** universe, worldoption_fmt * options,
-            double *acc_vh, double *acc_h, double *acc_w,
-            double *acc_c, long *vh, long *h, long *w, long *jj,
-            long *j, long *hot, long *warm, long *cold);
+run_loci(world_fmt ** universe, int usize, option_fmt * options,
+         data_fmt * data, tpool_t * heating_pool, long maxreplicate,
+         long *treefilepos, long *Gmax);
+
+void
+heated_swap(world_fmt ** universe, worldoption_fmt * options);
 
 void
 print_progress(worldoption_fmt * options, world_fmt * world,
@@ -155,9 +165,7 @@ prepare_next_chain(world_fmt ** universe, worldoption_fmt * options,
 void
 print_heating_progress(world_fmt ** universe,
                        worldoption_fmt * options,
-                       long steps, long increment, double *acc_vh,
-                       double *acc_h, double *acc_w, double *acc_c,
-                       long *hot, long *warm, long *cold);
+                       long stepinc);
 
 long
 analyze_olddata(world_fmt * world, option_fmt * options,
@@ -173,7 +181,7 @@ setup_locus(long locus, world_fmt * world, option_fmt * options,
 
 void
 condense_time(world_fmt * world, long *step, long *j,
-              long *accepted, long *G, long *steps,
+              double *accepted, long *G, long *steps,
               long oldsteps, long rep);
 
 long            set_repkind(option_fmt * options);
@@ -186,9 +194,7 @@ replicate_number(option_fmt * options, long chain, char type,
 
 void
 print_heating_progress2(FILE * file, worldoption_fmt * options,
-                        world_fmt ** universe, double acc_vh,
-                        double acc_h, double acc_w, double acc_c,
-                        long hot, long warm, long cold);
+                        world_fmt ** universe);
 
 
 void            get_treedata(world_fmt * world, option_fmt * options);
@@ -216,12 +222,13 @@ main(int argc, char **argv)
 #endif
 #ifdef MPI
     // parallel version
-	filenum = 0; //setting up the filename database so tht worker node can write back to master
+	filenum = 0; //setting up the filename database so that worker node can write back to master
     MPI_Init(&argc, &argv);
     comm_world = MPI_COMM_WORLD;
     MPI_Comm_size(comm_world, &numcpu);
+    fprintf(stdout,"==> %i <===\n",numcpu);
     MPI_Comm_rank(comm_world, &myID);
-    MPI_Barrier(comm_world);
+    MYMPIBARRIER(comm_world);
 #ifdef SLOWNET
     // slow network parallel version
     profiledone = 0;
@@ -231,6 +238,7 @@ main(int argc, char **argv)
     //scalar version of migrate
     myID = MASTER;
     numcpu = 1;
+fprintf(stdout,"==> SINGLE CPU VERSION <==\n");
 #endif
     //random number seed global inits
     seed = malloc(sizeof(long) * 3);
@@ -290,7 +298,7 @@ main(int argc, char **argv)
     if (options->writelog && myID==MASTER)
         print_menu_title(options->logfile, options);
     EARTH->repkind = SINGLECHAIN;
-    unset_penalizer_function(FALSE); // install penalizer for far jumps in MCMC
+    unset_penalizer_function(TRUE); // install penalizer for far jumps in MCMC
     if (!options->readsum)
     {
         //MCMC sampling
@@ -422,9 +430,9 @@ main(int argc, char **argv)
     mpi_send_stop(EARTH);
     //stop workers
     // printf("%i> at barrier in slownet-main before finalize\n", myID);
-    MPI_Barrier(comm_world);
-#endif
 
+#endif
+    MYMPIBARRIER(comm_world);
     MPI_Finalize();
 #endif
 
@@ -467,6 +475,7 @@ combine_loci(char type, option_fmt * options, world_fmt * world, long *Gmax)
         if (myID == MASTER)
         {
             mpi_gmax_master(world, Gmax);
+            mpi_startparam_master(world);
 #endif
 
             decide_plot(world->options, world->options->lchains,
@@ -481,6 +490,7 @@ combine_loci(char type, option_fmt * options, world_fmt * world, long *Gmax)
         {
             //printf("%i> before gmax worker\n", myID);
             mpi_gmax_worker(world);
+            mpi_startparam_worker(world);
             //printf("%i> start worker postlike calculations\n", myID);
             mpi_maximize_worker(world, options->replicatenum);
         }
@@ -631,253 +641,6 @@ run_loci(world_fmt ** universe, int usize, option_fmt * options,
     }
 }
 
-void
-run_locus(world_fmt ** universe, int usize, option_fmt * options,
-          data_fmt * data, tpool_t * heating_pool, long maxreplicate,
-          long locus, long *treefilepos, long *Gmax)
-{
-    long            kind;
-    long            replicate;
-    char            type;
-    long            runs, rep, pluschain, increment, oldsteps, chains;
-    long            chain = 0, hot, warm, cold, accepted, steps, step,
-                            i, ii, j, jj;
-    double          acc_vh = 0.0, acc_h = 0.0, acc_w = 0.0, acc_c = 0.0;
-    long            vh = 0, h = 0, w = 0;
-    /* loop over independent replicates----------------------- */
-    /* but make sure that we get single chain estimates, too   */
-    EARTH->locus = locus;
-    EARTH->repkind = SINGLECHAIN;
-    for (replicate = 0; replicate < maxreplicate; replicate++)
-    {
-        EARTH->replicate = replicate;
-        if (setup_locus(locus, EARTH, options, data) == 0)
-            continue;
-        if (options->heating)
-            heating_prepare(universe, usize, options, data, replicate);
-        type = 's';
-        runs = 1;
-        rep = 0;
-        pluschain = options->pluschain;
-        //see obscure options
-        /* short and long chains ----------------------------- */
-        set_bounds(&increment, &oldsteps, &chains, options, type);
-        EARTH->increment = increment;
-        while (runs-- >= 0)
-        {
-            if (myID == MASTER)
-            {
-                print_menu_chain(type, FIRSTCHAIN, oldsteps, EARTH,
-                                 options, rep);
-                if (EARTH->options->treeprint == ALL)
-                    print_tree(EARTH, 0, treefilepos);
-            }
-            EARTH->chains = chains;
-            /* loop over chains--------------------------- */
-            for (chain = 0;
-                    chain < chains || (type == 'l' && chain >= chains
-                                       && EARTH->param_like >
-                                       options->lcepsilon); chain++)
-            {
-                hot = warm = cold = 0;
-                rep = replicate_number(options, chain, type, replicate);
-                precalc_world(EARTH);
-                polish_world(EARTH);
-                burnin_chain(EARTH);
-                EARTH->G = 0;
-                accepted = 0;
-                steps = oldsteps;
-                EARTH->maxdatallike = EARTH->likelihood[0];
-                if (options->heating)
-                    heating_prepare2(universe, usize);
-                if (type == 'l')
-                {
-                    if (chain >= chains - 1)
-                    {
-                        EARTH->in_last_chain = TRUE;
-                        unset_penalizer_function(TRUE);
-                    }
-                }
-                else
-                {
-                    EARTH->in_last_chain = FALSE;
-                    unset_penalizer_function(FALSE);
-                }
-                /*
-                 * steps for each chain
-                 * ------------------------
-                 */
-                for (step = 0; step < steps + 1; step++)
-                {
-                    j = 0;
-                    EARTH->increment = increment;
-                    /*
-                     * hop over INCREMENT trees,sample
-                     * last
-                     */
-                    for (i = 0; i < increment; i++)
-                    {
-                        EARTH->actualinc = i;
-                        //runclones of world at different
-                        // temperatures
-                        if (options->heating)
-                        {
-#ifdef PTHREADS
-                            fill_tpool(*heating_pool, universe, usize);
-                            tpool_synchronize(*heating_pool, usize);
-                            acc_vh += (vh = SUN->accept);
-                            acc_h += (h = MERKUR->accept);
-                            acc_w += (w = VENUS->accept);
-                            jj = EARTH->accept;
-#else
-
-                            for (ii = HEATED_CHAIN_NUM;
-                                    ii < options->heated_chains; ii++)
-                            {
-#ifdef BAYESUPDATE
-                                if (universe[ii]->options->bayes_infer)
-                                    universe[ii]->bayesaccept = bayes_update(universe[ii]);
-#endif
-
-                                universe[ii]->accept =
-                                    tree_update(universe[ii], universe[ii]->G);
-                            }
-#ifdef BAYESUPDATE
-                            if (EARTH->options->bayes_infer)
-                            {
-                                SUN->bayesaccept = bayes_update(SUN);
-                                MERKUR->bayesaccept = bayes_update(MERKUR);
-                                VENUS->bayesaccept = bayes_update(VENUS);
-                                EARTH->bayesaccept = bayes_update(EARTH);
-                            }
-#endif
-                            acc_vh += (vh = tree_update(SUN, SUN->G));
-                            acc_h += (h = tree_update(MERKUR, MERKUR->G));
-                            acc_w += (w = tree_update(VENUS, VENUS->G));
-
-                            jj = tree_update(EARTH, EARTH->G);
-#endif
-#ifdef UEP
-
-                            if (options->uep && EARTH->in_last_chain)
-                                update_uepanc(EARTH);
-#endif
-
-                            heated_swap(universe, EARTH->options, &acc_vh,
-                                        &acc_h, &acc_w, &acc_c, &vh, &h, &w,
-                                        &jj, &j, &hot, &warm, &cold);
-                            if(options->adaptiveheat)
-                                adjust_temperatures(universe, options->heated_chains, i/*ncrement*/+step*increment, steps*increment);
-
-                        }
-                        else
-                        {
-#ifdef BAYESUPDATE
-                            if (EARTH->options->bayes_infer)
-                                EARTH->bayesaccept = bayes_update(EARTH);
-#endif
-
-                            j += tree_update(EARTH, EARTH->G);
-#ifdef UEP
-
-                            if (options->uep && EARTH->in_last_chain)
-                                update_uepanc(EARTH);
-#endif
-
-                        }
-                        if (EARTH->likelihood[EARTH->G] > EARTH->maxdatallike)
-                        {
-                            EARTH->maxdatallike = EARTH->likelihood[EARTH->G];
-                        }
-                    }
-                    if (options->heating)
-                    {
-                        advance_clone_like(VENUS, w, &VENUS->G);
-                        advance_clone_like(MERKUR, h, &MERKUR->G);
-                        advance_clone_like(SUN, vh, &SUN->G);
-                        for (ii = HEATED_CHAIN_NUM;
-                                ii < EARTH->options->heated_chains; ++ii)
-                            advance_clone_like(universe[ii],
-                                               universe[ii]->accept,
-                                               &(universe[ii])->G);
-                    }
-                    /*
-                     * store condensed tree information
-                     * for estimation
-                     */
-#ifdef UEP
-                    store_uep(EARTH);
-#endif
-
-                    condense_time(EARTH, &step, &j, &accepted, &EARTH->G,
-                                  &steps, oldsteps, rep);
-                    if (step > 0)
-                        advance_clone_like(EARTH, j, &EARTH->G);
-                } //end steps
-                decide_plot(EARTH->options, chain, chains, type);
-                memcpy(EARTH->param00, EARTH->param0,
-                       sizeof(double) * EARTH->numpop2);
-                memcpy(EARTH->atl[rep][locus].param0, EARTH->param00,
-                       sizeof(double) * EARTH->numpop2);
-                log_param0(EARTH->atl[rep][locus].param0,
-                           EARTH->atl[rep][locus].lparam0, EARTH->numpop2);
-                copies2lcopies(&EARTH->atl[rep][locus]);
-                EARTH->repkind = SINGLECHAIN;
-                //single chain, single replicate
-                kind = SINGLELOCUS;
-                *Gmax = (EARTH->G > *Gmax) ? EARTH->G + 1 : (*Gmax);
-//#ifdef BAYESUPDATE
-//
-//                if (options->bayes_infer)
-//                    bayes_stat(EARTH);
-//                else
-//#endif
-
-                    estimateParameter(rep, *Gmax, EARTH, options,
-                                      EARTH->cov[locus], chain, type, kind,
-                                      EARTH->repkind);
-                //print_progress(EARTH->options, EARTH, rep, steps * increment,
-                 //              accepted);
-                prepare_next_chain(universe, EARTH->options, type, chain,
-                                   &chains, &pluschain, locus, replicate);
-                print_heating_progress(universe, EARTH->options, steps,
-                                       increment, &acc_vh, &acc_h, &acc_w,
-                                       &acc_c, &hot, &warm, &cold);
-            } //end chains
-            change_chaintype(locus, &type, EARTH, &increment, &oldsteps,
-                             &chains, options);
-            /* evaluate multiple long chain estimates */
-            if (runs < 0 && options->replicate && options->replicatenum == 0)
-            {
-                EARTH->repkind = MULTIPLECHAIN;
-                estimateParameter(rep, *Gmax, EARTH, options,
-                                  EARTH->cov[locus], chain, type,
-                                  SINGLELOCUS, EARTH->repkind);
-            }
-        } //end runs
-    } //end replicates
-#ifdef UEP
-    if (options->uep)
-        show_uep_store(EARTH);
-#endif
-
-    if (options->replicate && options->replicatenum > 0)
-    {
-        EARTH->repkind = MULTIPLERUN;
-#ifdef BAYESUPDATE
-
-        if (options->bayes_infer)
-        {
-           // bayes_stat(EARTH);
-            return;
-        }
-#endif
-        estimateParameter(options->replicatenum, *Gmax, EARTH, options,
-                          EARTH->cov[locus], chain, type,
-                          SINGLELOCUS, EARTH->repkind);
-    }
-}
-
 /* save all genealogy-summaries --------------------------- */
 void
 get_treedata(world_fmt * world, option_fmt * options)
@@ -936,62 +699,24 @@ get_mighistdata(world_fmt * world, option_fmt * options)
 }
 
 void
-heated_swap(world_fmt ** universe, worldoption_fmt * options, double *acc_vh,
-            double *acc_h, double *acc_w, double *acc_c, long *vh, long *h,
-            long *w, long *jj, long *j, long *hot, long *warm, long *cold)
+heated_swap(world_fmt ** universe, worldoption_fmt * options)
 {
     long            sm = 0, mv = 0, vw = 0;
     long            r;
     switch (r = RANDINT(0, options->heated_chains - 2))
     {
     case 2:
-        MERKUR->treeswapcount += (sm = chance_swap_tree(SUN, MERKUR));
-        mv = vw = 0;
-        *vh += sm;
-        *h += sm;
-        *acc_vh += sm;
-        *acc_h += sm;
-        break;
+      sm = chance_swap_tree(SUN, MERKUR);
+      MERKUR->swapped += sm; 
+      break;
     case 1:
-        VENUS->treeswapcount += (mv = chance_swap_tree(MERKUR, VENUS));
-        sm = vw = 0;
-        *h += mv;
-        *w += mv;
-        *acc_h += mv;
-        *acc_w += mv;
+        VENUS->swapped += (mv = chance_swap_tree(MERKUR, VENUS));
         break;
     case 0:
-        EARTH->treeswapcount += (vw = chance_swap_tree(VENUS, EARTH));
-        *acc_w += vw;
-        *w += vw;
-        mv = sm = 0;
+        EARTH->swapped += (vw = chance_swap_tree(VENUS, EARTH));
         break;
     default:
-        universe[r]->treeswapcount += chance_swap_tree(universe[r+1], universe[r]);
-        sm = vw = mv = 0;
-    }
-    *jj += vw;
-    /*
-     * if(options->verbose) { printf("   Boil%c%c%5.5f ",(int)(*vh)+' ',
-     * sm==0?':':'*', SUN->likelihood[SUN->G]); printf("Hot %c%c%5.5f
-     * ",(int)(*h)+' ', sm==0 && mv==0 ?':':'*',
-     * MERKUR->likelihood[MERKUR->G]); printf("Warm%c%c%5.5f
-     * ",(int)(*w)+' ', mv==0 && vw==0?':':'*',
-     * VENUS->likelihood[VENUS->G]);
-     * printf("Cold%c%c%5.5f\n",(int)(*jj)+' ', vw==0?':':'*',
-     * EARTH->likelihood[EARTH->G]);
-     * 
-     * } */
-    if (options->progress && options->heating)
-    {
-        *hot += sm;
-        *warm += mv;
-        *cold += vw;
-    }
-    if (*jj > 0)
-    {
-        (*j)++;
-        (*acc_c)++;
+        universe[r]->swapped += chance_swap_tree(universe[r+1], universe[r]);
     }
 }
 
@@ -1010,6 +735,7 @@ print_progress(worldoption_fmt * options, world_fmt * world, long rep,
         if (!world->start && world->chains > 1 && options->verbose)
             print_gelmanr(world->gelmanmeanR, world->gelmanmaxR);
     }
+    world->accept_freq = world->accept = 0;
 }
 
 void
@@ -1062,32 +788,40 @@ prepare_next_chain(world_fmt ** universe, worldoption_fmt * options,
 
 void
 print_heating_progress(world_fmt ** universe, worldoption_fmt * options,
-                       long steps, long increment, double *acc_vh,
-                       double *acc_h, double *acc_w, double *acc_c,
-                       long *hot, long *warm, long *cold)
+                       long stepinc)
 {
-    if (options->heating)
+
+  if (options->heating)
     {
-        *acc_vh /= increment * steps;
-        *acc_h /= increment * steps;
-        *acc_w /= increment * steps;
-        *acc_c /= increment * steps;
-        if (options->progress)
-            print_heating_progress2(stdout, options, universe, *acc_vh, *acc_h,
-                                    *acc_w, *acc_c, *hot, *warm, *cold);
-        if (options->writelog)
-            print_heating_progress2(options->logfile, options, universe, *acc_vh,
-                                    *acc_h, *acc_w, *acc_c, *hot, *warm, *cold);
-        *hot = *warm = *cold = 0;
-        *acc_vh = *acc_h = *acc_w = *acc_c = 0.0;
+      universe[0]->accept_freq /= stepinc;
+      universe[1]->accept_freq /= stepinc;
+      universe[2]->accept_freq /= stepinc;
+      universe[3]->accept_freq /= stepinc;
+      if (options->progress)
+          print_heating_progress2(stdout, options, universe);
+      if (options->writelog)
+          print_heating_progress2(options->logfile, options, universe);
+      if(options->heated_chains > 4)
+          universe[3]->swapped = 0;
+      universe[2]->swapped = 0;
+      universe[1]->swapped = 0;
+      universe[0]->swapped = 0;
+
+      universe[0]->accept = 0.;
+      universe[1]->accept = 0.;
+      universe[2]->accept = 0.;
+      universe[3]->accept = 0.;
+
+      universe[0]->accept_freq = 0.;
+      universe[1]->accept_freq = 0.;
+      universe[2]->accept_freq = 0.;
+      universe[3]->accept_freq = 0.;
     }
 }
 
 void
 print_heating_progress2(FILE * file, worldoption_fmt * options,
-                        world_fmt ** universe, double acc_vh, double acc_h,
-                        double acc_w, double acc_c, long hot, long warm,
-                        long cold)
+                        world_fmt ** universe)
 {
     FPRINTF(file, "           Swapping between %li temperatures.\n",
             options->heated_chains);
@@ -1095,14 +829,18 @@ print_heating_progress2(FILE * file, worldoption_fmt * options,
         FPRINTF(file, "               Only the 4 coldest chains are shown\n");
     FPRINTF(file,
             "           Temperature | Accepted |  Swaps between temperatures\n");
-    FPRINTF(file, "           %10.4f  |   %4.2f   |          |\n",
-            universe[3]->averageheat, acc_vh);
-    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li||\n",
-            universe[2]->averageheat, acc_h, hot);
-    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li ||\n",
-            universe[1]->averageheat, acc_w, warm);
-    FPRINTF(file, "           %10.4f  |   %4.2f   |     %5li  |\n", universe[0]->averageheat,
-            acc_c, cold);
+    if(options->heated_chains > 4)
+      FPRINTF(file, "           %10.4f  |   %4.2f   |    %5li||\n",
+	      universe[3]->averageheat, universe[3]->accept_freq,universe[3]->swapped);
+    else
+      FPRINTF(file, "           %10.4f  |   %4.2f   |         |\n",
+	      universe[3]->averageheat, universe[3]->accept_freq);
+    FPRINTF(file, "           %10.4f  |   %4.2f   |    %5li ||\n",
+            universe[2]->averageheat, universe[2]->accept_freq, universe[2]->swapped);
+    FPRINTF(file, "           %10.4f  |   %4.2f   |    %5li  ||\n",
+            universe[1]->averageheat, universe[1]->accept_freq, universe[1]->swapped);
+    FPRINTF(file, "           %10.4f  |   %4.2f   |    %5li   |\n", universe[0]->averageheat,
+            universe[0]->accept_freq, universe[0]->swapped);
 }
 
 long
@@ -1117,19 +855,28 @@ analyze_olddata(world_fmt * world,
 #endif
 
     long            Gmax = 0;
+    long pop;
 	options->heating = FALSE;
     if (myID == MASTER)
     {
         unset_penalizer_function(TRUE);
         Gmax = read_savesum(world, options, data);
+
+        data->popnames = (char **) malloc (sizeof (char *) * world->numpop);
+        for (pop = 0; pop < world->numpop; pop++)
+        {
+            data->popnames[pop] = (char *) calloc (1, sizeof (char) * LINESIZE);
+            sprintf(data->popnames[pop],"Pop_%li ",pop+1); 
+        }
+        
 #ifdef MPI
 		
-        MPI_Bcast(data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
-        MPI_Bcast(data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+        MYMPIBCAST(data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+        MYMPIBCAST(data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
     }
     else
     {
-        MPI_Bcast(listofthings, 6, MPI_LONG, MASTER, comm_world);
+        MYMPIBCAST(listofthings, 6, MPI_LONG, MASTER, comm_world);
         //      printf("listofthings received\n");
         //    fflush(stdout);
         world->loci = listofthings[0];
@@ -1141,8 +888,9 @@ analyze_olddata(world_fmt * world,
         data->numpop = world->numpop;
         data->geo = (double *) calloc(1, sizeof(double) * world->numpop2);
         data->lgeo = (double *) calloc(1, sizeof(double) * world->numpop2);
-        MPI_Bcast(data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
-        MPI_Bcast(data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+
+        MYMPIBCAST(data->geo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
+        MYMPIBCAST(data->lgeo, world->numpop2, MPI_DOUBLE, MASTER, comm_world);
         options->muloci = data->loci = world->loci;
         init_world(world, data, options);
 #endif
@@ -1225,7 +973,7 @@ profile_tables(option_fmt * options, world_fmt * world, long *gmaxptr)
             mpi_broadcast_results(world, world->loci,
                                   pack_sumfile_buffer,
                                   unpack_sumfile_buffer);
-        MPI_Barrier(comm_world);
+        MYMPIBARRIER(comm_world);
         mpi_broadcast_results(world, world->loci + ((world->loci == 1) ? 0 : 1),
                               pack_result_buffer,
                               unpack_result_buffer);
@@ -1334,12 +1082,13 @@ setup_locus(long locus, world_fmt * world, option_fmt * options,
 }
 
 void
-condense_time(world_fmt * world, long *step, long *j, long *accepted,
+condense_time(world_fmt * world, long *step, long *j, double *accepted,
               long *G, long *steps, long oldsteps, long rep)
 {
 
 #ifdef BAYESUPDATE
-    if (world->in_last_chain && world->options->bayes_infer && *accepted)
+    if (world->in_last_chain && world->options->bayes_infer /*&& 
+	(((1+world->bayes->count) % ((long) world->options->updateratio)) == 0)*/ /**accepted*/)
         bayes_save(world);
 #endif
 
@@ -1402,7 +1151,7 @@ heating_prepare(world_fmt ** universe, int usize,
                 option_fmt * options, data_fmt * data, long rep)
 {
     long            chain;
-    EARTH->heat[0] = options->heat[0];
+    EARTH->heat = options->heat[0];
     if (rep == 0)
     {
         for (chain = 1; chain < usize; ++chain)
@@ -1425,7 +1174,7 @@ heating_prepare2(world_fmt ** universe, int usize)
         universe[chain]->G = 0;
         universe[chain]->averageheat = universe[chain]->options->adaptiveheat ?
                                        0.0 :
-                                       1./universe[chain]->heat[0];
+                                       1./universe[chain]->heat;
     }
     for (chain = 1; chain < usize; ++chain)
     {
@@ -1455,3 +1204,294 @@ replicate_number(option_fmt * options, long chain, char type, long replicate)
     }
     return rep;
 }
+void
+run_locus(world_fmt ** universe, int usize, option_fmt * options,
+          data_fmt * data, tpool_t * heating_pool, long maxreplicate,
+          long locus, long *treefilepos, long *Gmax)
+{
+    long            replicate;
+    for (replicate = 0; replicate < maxreplicate; replicate++)
+        run_replicate(locus, replicate, universe, options, data, 
+		      heating_pool, usize,
+		      treefilepos, Gmax);
+    
+#ifdef UEP
+    if (options->uep)
+        show_uep_store(EARTH);
+#endif
+    if (options->replicate && options->replicatenum > 0)
+    {
+        EARTH->repkind = MULTIPLERUN;
+#ifdef BAYESUPDATE
+        if (options->bayes_infer)
+            return;
+#endif
+        // over multiple replicates if present or single locus
+        estimateParameter(options->replicatenum, *Gmax, EARTH, options,
+                          EARTH->cov[locus], options->lchains, /*type*/ 'l',
+                          SINGLELOCUS, EARTH->repkind);
+    }
+}
+
+
+void run_updates(world_fmt ** universe,
+                int usize,
+                option_fmt * options,
+                tpool_t * heating_pool,
+                long inc,
+                long increment,
+                long step,
+                long steps)
+{
+    long ii;
+    if (options->heating)
+    {
+#ifdef PTHREADS /*using threads and running on an SMP machine*/
+        fill_tpool(*heating_pool, universe, usize);
+        tpool_synchronize(*heating_pool, usize);
+#else  /*heating but not using threads*/
+        for (ii = 0; ii < options->heated_chains; ii++)
+        {
+#ifdef BAYESUPDATE
+            if (universe[ii]->options->bayes_infer)
+                universe[ii]->bayesaccept = bayes_update(universe[ii]);
+#endif
+            universe[ii]->accept +=
+                tree_update(universe[ii], universe[ii]->G);
+        }
+#endif /* end of not-using threads*/
+        heated_swap(universe, EARTH->options);
+        if(options->adaptiveheat)
+            adjust_temperatures(universe, options->heated_chains, inc/*rement*/+step*increment, steps*increment);        
+    }
+    else /* no heating*/
+      {
+#ifdef BAYESUPDATE
+        if (EARTH->options->bayes_infer)
+            EARTH->bayesaccept = bayes_update(EARTH);
+#endif
+        EARTH->accept +=
+            tree_update(EARTH, EARTH->G);        
+      }
+#ifdef UEP
+    if (options->uep && EARTH->in_last_chain)
+        update_uepanc(EARTH);
+#endif
+}
+
+void
+run_increments(world_fmt **universe,
+               long usize,
+               option_fmt *options,
+               tpool_t * heating_pool,
+               long increment,
+               long step,
+               long steps)
+{
+    long i;
+    for (i = 0; i < increment; i++)
+    {
+        EARTH->actualinc = i;
+        run_updates(universe, usize, options,
+                    heating_pool, i,increment,
+                    step, steps);
+    
+        if (EARTH->likelihood[EARTH->G] > EARTH->maxdatallike)
+            EARTH->maxdatallike = EARTH->likelihood[EARTH->G];
+    }
+}
+
+void
+run_steps(world_fmt **universe,
+               long usize,
+               option_fmt *options,
+               tpool_t * heating_pool,
+               long increment,
+               long steps,
+                long rep)
+{
+    long step;
+    long oldsteps = steps;
+    long ii;
+    for (step = 0; step < steps + 1; step++)
+    {
+        EARTH->increment = increment;
+        run_increments(universe,usize,options,heating_pool,
+                    increment, step, steps);
+    #ifdef UEP
+        store_uep(EARTH);
+    #endif
+        condense_time(EARTH, &step, &EARTH->accept,
+                    &EARTH->accept_freq,
+                    &EARTH->G,
+                    &steps, oldsteps, rep);
+        if (step > 0)
+          {
+            advance_clone_like(EARTH, EARTH->accept,
+                            &EARTH->G);
+            EARTH->accept = 0;
+          }
+        // if we do heating
+        if(EARTH->options->heating)
+          {
+            for (ii = 1;
+                ii < EARTH->options->heated_chains; ++ii)
+              {
+                universe[ii]->accept_freq += universe[ii]->accept;
+                advance_clone_like(universe[ii],
+                                universe[ii]->accept,
+                                &(universe[ii])->G);
+                universe[ii]->accept = 0;
+              }
+          }
+    }
+}
+
+
+void run_chains(world_fmt **universe,
+                long usize,
+                option_fmt *options,
+                tpool_t * heating_pool,
+                long replicate,
+                long chains,
+                char type,
+                long increment,
+                long oldsteps,
+		long *treefilepos, 
+		long *Gmax)
+{
+    long steps;
+    long chain;
+    long rep;
+    long locus = EARTH->locus;
+    long kind;
+    long pluschain=0;
+    for (chain = 0;
+        chain < chains || (type == 'l' && chain >= chains
+                            && EARTH->param_like >
+                            options->lcepsilon); chain++)
+    {
+        if(options->heating)
+            MERKUR->swapped = VENUS->swapped = EARTH->swapped = 0;
+        rep = replicate_number(options, chain, type, replicate);
+        precalc_world(EARTH);
+        polish_world(EARTH);
+        burnin_chain(EARTH);
+        EARTH->G = 0;
+        EARTH->accept_freq = 0.;
+        steps = oldsteps;
+        EARTH->maxdatallike = EARTH->likelihood[0];
+        if (options->heating)
+            heating_prepare2(universe, usize);
+
+        set_penalizer(chain, chains, type, universe);
+        // run steps, run increments, run updaters 
+        run_steps(universe,usize,options,heating_pool,
+		  increment, steps, rep);
+
+        decide_plot(EARTH->options, chain, chains, type);
+        // prepare for parameter estimation, precompute and copy
+        memcpy(EARTH->param00, EARTH->param0,
+            sizeof(double) * EARTH->numpop2);
+        memcpy(EARTH->atl[rep][locus].param0, EARTH->param00,
+            sizeof(double) * EARTH->numpop2);
+        log_param0(EARTH->atl[rep][locus].param0,
+                EARTH->atl[rep][locus].lparam0, EARTH->numpop2);
+        copies2lcopies(&EARTH->atl[rep][locus]);
+        // start parameter estimation for 
+        // single chain, single replicate
+        EARTH->repkind = SINGLECHAIN;
+        kind = SINGLELOCUS;
+        *Gmax = (EARTH->G > *Gmax) ? EARTH->G + 1 : (*Gmax);
+        estimateParameter(rep, *Gmax, EARTH, options,
+                        EARTH->cov[locus], chain, type, kind,
+                        EARTH->repkind);
+        if(EARTH->options->heating)
+            print_heating_progress(universe, EARTH->options, increment * steps);
+        else
+            print_progress(EARTH->options,EARTH,rep, 
+                           increment * steps, EARTH->accept_freq);
+        // cleanup and prepare next
+        prepare_next_chain(universe, EARTH->options, type, chain,
+                           &chains, &pluschain, locus, replicate);
+    } //end chains
+}
+
+void set_penalizer(long chain, long chains, char type, world_fmt **universe)
+{
+    if (type == 'l')
+    {
+        if (chain >= chains - 1)
+        {
+            EARTH->in_last_chain = TRUE;
+            unset_penalizer_function(TRUE);
+        }
+    }
+    else
+    {
+        EARTH->in_last_chain = FALSE;
+        unset_penalizer_function(TRUE);
+    }
+}
+
+void run_replicate(long locus,
+                   long replicate,
+                   world_fmt **universe,
+                   option_fmt *options,
+                   data_fmt *data, 
+		   tpool_t * heating_pool,
+		   int usize,
+		   long *treefilepos,
+		   long *Gmax)
+{
+    long chains=0;
+    char type='s';
+    long pluschain;
+    long runs;
+    long increment=1;
+    long steps;
+    long oldsteps=0;
+    /* loop over independent replicates----------------------- */
+    /* but make sure that we get single chain estimates, too   */
+    EARTH->locus = locus;
+    EARTH->repkind = SINGLECHAIN;
+    EARTH->replicate = replicate;
+    if (setup_locus(locus, EARTH, options, data) == 0)
+        return;
+    if (options->heating)
+            heating_prepare(universe, usize, options, data, replicate);
+        type = 's';
+        runs = 1;
+       // rep = 0;
+        pluschain = options->pluschain;
+        //see obscure options
+        /* short and long chains ----------------------------- */
+        set_bounds(&increment, &oldsteps, &chains, options, type);
+        steps = oldsteps;
+        EARTH->increment = increment;
+        while (runs-- >= 0)
+          {
+            if (myID == MASTER)
+              {
+                print_menu_chain(type, FIRSTCHAIN, oldsteps, EARTH,
+                                 options, replicate);
+                if (EARTH->options->treeprint == ALL)
+                    print_tree(EARTH, 0, treefilepos);
+              }
+            EARTH->chains = chains;
+            /* loop over chains--------------------------- */
+            run_chains(universe, usize,
+                       options, heating_pool, replicate, chains, type,
+                       increment, oldsteps, treefilepos, Gmax);//PB 020203
+            change_chaintype(locus, &type, EARTH, &increment, &oldsteps,
+                             &chains, options);
+            /* evaluate multiple long chain estimates */
+            if (runs < 0 && options->replicate && options->replicatenum == 0)
+              {
+                EARTH->repkind = MULTIPLECHAIN;
+                estimateParameter(replicate, *Gmax, EARTH, options,
+                                  EARTH->cov[locus], chains, type,
+                                  SINGLELOCUS, EARTH->repkind);
+              }
+          } //end runs
+}
