@@ -13,13 +13,13 @@
  
 Copyright 2001 Peter Beerli and Joseph Felsenstein
  
-$Id: menu.h,v 1.7 2002/06/20 06:25:36 beerli Exp $
+$Id: menu.h,v 1.8 2002/08/12 21:13:54 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
 
 extern void print_menu_title (FILE * file, option_fmt * options);
-extern void print_menu_accratio (FILE * file, long a, long b);
+extern void print_menu_accratio (long a, long b, world_fmt *world);
 extern long print_title (world_fmt * world, option_fmt * options);
 extern void get_menu (option_fmt * options);
 
