@@ -10,9 +10,10 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: migrate_mpi.c,v 1.22 2002/08/12 23:41:40 beerli Exp $
+$Id: migrate_mpi.c,v 1.25 2002/11/14 18:38:04 beerli Exp $
 */
 #ifdef MPI
+#include "sighandler.h"
 #include "migrate_mpi.h"
 #include "broyden.h"
 #include "combroyden.h"
@@ -74,7 +75,7 @@ mpi_runloci_master (long loci, int *who, world_fmt *world)
 
     for (locus = 0; locus < MIN (loci, numcpu - 1); locus++)
     {
-        MPI_Send (&locus, 1, MPI_LONG, locus + 1, locus + 1, comm_world);
+        MYMPISEND (&locus, 1, MPI_LONG, locus + 1, locus + 1, comm_world);
         numsent++;
     }
     for (locus = 0; locus < loci; locus++)
@@ -82,10 +83,9 @@ mpi_runloci_master (long loci, int *who, world_fmt *world)
         done=FALSE;
         while(!done)
         {
-            MPI_Recv (rawmessage, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG,
-                      comm_world, &status);
+            MYMPIRECV (rawmessage, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE,MPI_ANY_TAG, comm_world, &status);
             sender = status.MPI_SOURCE;
-            //			fprintf(stdout,"%i> @@@@%s@@@@\n",sender, rawmessage);
+//            fprintf(stdout,"%i> @@@@%s@@@@\n",sender, rawmessage);
             switch(rawmessage[0])
             {
             case 'M':
@@ -106,19 +106,19 @@ mpi_runloci_master (long loci, int *who, world_fmt *world)
         who[locusdone] = sender;
         if (numsent < loci)
         {
-            MPI_Send (&numsent, 1, MPI_LONG, sender, numsent + 1, comm_world);
+            MYMPISEND (&numsent, 1, MPI_LONG, sender, numsent + 1, comm_world);
             numsent++;
         }
         else
         {
-            MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker to wait for new loci
+            MYMPISEND (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop workers to wait for new loci
         }
     }
     //stop worker that had never the chance to work on a locus, but are still
     //listening
     for (sender = MIN (loci, numcpu - 1) + 1; sender < numcpu; sender++)
     {
-        MPI_Send (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker to wait for new loci
+        MYMPISEND (&loci, 1, MPI_LONG, sender, 0, comm_world); //stop worker to wait for new loci
     }
 }
 
@@ -137,14 +137,14 @@ mpi_runloci_worker (world_fmt ** universe, int usize,
     rawmessage = (char *) calloc(STRSIZE,sizeof(char));
     while (!done)
     {
-        MPI_Recv (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
+        MYMPIRECV (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
                   comm_world, &status);
         if (status.MPI_TAG != 0) //stop condition
         {
             run_locus (universe, usize, options, data,
                        heating_pool, maxreplicate, locus, treefilepos, Gmax);
             rawmsgsize = sprintf(rawmessage,"R%li",locus) + 1;
-            MPI_Send (rawmessage, rawmsgsize, MPI_CHAR, MASTER, locus + 1, comm_world);
+            MYMPISEND (rawmessage, rawmsgsize, MPI_CHAR, MASTER, locus + 1, comm_world);
             /* we want to know what locus we worked for
                - to control the work sent by master
                - to use in setup_parameter0() [combroyden2.c] */
@@ -157,6 +157,130 @@ mpi_runloci_worker (world_fmt ** universe, int usize,
     }
 }
 
+#if 0
+//---------replication in MPI
+void
+mpi_runreplicates_master (long loci, int *who, world_fmt *world)
+{
+    long locrep;
+    int sender = 0;
+    long locusdone = -1;
+    MPI_Status status;
+    long numsent = 0;
+    boolean done = FALSE;
+    char *rawmessage;
+    long temp[2];
+    long  maxreplicate = (world->options->replicate
+                          && world->options->replicatenum >
+                          0) ? world->options->replicatenum : 1;
+    long locirep = loci * maxreplicate;
+    
+    rawmessage = (char *) calloc(MAXBUFSIZE,sizeof(char));
+    // loop over loci and replicates, first loop is only over numcpu 
+    for (locrep = 0; locrep < MIN (locirep, numcpu - 1); locrep++)
+      {
+        temp[0] = (long) (locrep / maxreplicate); // locus to work on
+        temp[1] = (long) (locrep % maxreplicate;  // replicate to work with
+        MYMPISEND (temp, 2, MPI_LONG, locrep + 1, locrep + 1, comm_world);
+        numsent++;
+      }
+    // loope over all loci and replicates from numcpu to loci*replicates
+    for (locrep = 0; locrep < locirep; locrep++)
+      {
+        done=FALSE;
+        while(!done)
+          {
+            MYMPIRECV (rawmessage, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG,
+                      comm_world, &status);
+            sender = status.MPI_SOURCE;
+            switch(rawmessage[0])
+              {
+                case 'M':
+                    sender = status.MPI_SOURCE;
+                    handle_message(rawmessage,sender, world);
+                    break;
+                case 'R':
+                    //ignore first character and translate into locusnumber
+                    sscanf(rawmessage,"R%li:%li",&locusdone,$replicatedone);
+                    sender = status.MPI_SOURCE;
+                    done=TRUE;
+                    break;
+                default:
+                    error("DIED because of wrong message from worker");
+                    break;
+              }
+          }
+        who[locusdone][0] = sender;
+        who[locusdone][1] = replicatedone;
+        if (numsent < locirep)
+          {
+            temp[0] = (long) (numsent / maxreplicate); // locus to work on
+            temp[1] = (long) (numsent % maxreplicate;  // replicate to work with
+            MYMPISEND (temp, 2, MPI_LONG, sender, numsent + 1, comm_world);
+            numsent++;
+          }
+        else
+          {
+            temp[0] = loci;
+            temp[1] = -1;
+            //tell workers to stop waiting for new loci
+            MYMPISEND (temp, 2, MPI_LONG, sender, 0, comm_world);
+          }
+      }
+    //tell worker who had never the chance to work on a locus, but are still
+    //listening, that they can stop waiting
+    temp[0] = loci;
+    temp[1] = -1;
+    for (sender = MIN (locirep, numcpu - 1) + 1; sender < numcpu; sender++)
+      {
+        MYMPISEND (temp, 2, MPI_LONG, sender, 0, comm_world);
+      }
+}
+
+
+void
+mpi_runreplicates_worker (world_fmt ** universe, int usize,
+                    option_fmt * options, data_fmt * data,
+                    tpool_t * heating_pool, long maxreplicate,
+                    long *treefilepos, long *Gmax)
+{
+    boolean done = FALSE;
+    long locus;
+    char *rawmessage;
+    long rawmsgsize = 0;
+    MPI_Status status;
+    rawmessage = (char *) calloc(STRSIZE,sizeof(char));
+    while (!done)
+      {
+        MYMPIRECV (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
+                  comm_world, &status);
+        if (status.MPI_TAG != 0) //stop condition
+          {
+            run_locus (universe, usize, options, data,
+                       heating_pool, maxreplicate, locus, treefilepos, Gmax);
+            rawmsgsize = sprintf(rawmessage,"R%li",locus) + 1;
+            MYMPISEND (rawmessage, rawmsgsize, MPI_CHAR, MASTER, locus + 1, comm_world);
+            /* we want to know what locus we worked for
+                - to control the work sent by master
+                - to use in setup_parameter0() [combroyden2.c] */
+            universe[0]->who[locidone++] = locus;
+          }
+        else
+          {
+            done = TRUE;
+          }
+      }
+    // transfer data to sub-master for summarizing
+    // receive from master to whom I need to send locus/replicate
+    // send replicate parts to sub-master
+    // if submaster calculate over replicates
+    // if submaster signal ready to master so that master can update who records
+}
+
+//---------end replication in MPI
+#endif
+
+
 double
 mpi_likelihood_master (double *param, double *lparam,
                        world_fmt * world, nr_fmt * nr,
@@ -185,11 +309,11 @@ mpi_likelihood_master (double *param, double *lparam,
         addon=1;
     for (worker = 1; worker < MIN (world->loci + addon, numcpu); worker++)
     {
-        MPI_Send (temp, (int) numelem2+2, MPI_DOUBLE, worker, worker, comm_world);
+        MYMPISEND (temp, (int) numelem2+2, MPI_DOUBLE, worker, worker, comm_world);
     }
     for (worker = 1; worker < MIN (world->loci + addon, numcpu); worker++)
     {
-        MPI_Recv (tmp, (int) world->loci, MPI_DOUBLE, MPI_ANY_SOURCE,
+        MYMPIRECV (tmp, (int) world->loci, MPI_DOUBLE, MPI_ANY_SOURCE,
                   MPI_ANY_TAG, comm_world, &status);
         sender = status.MPI_SOURCE;
         tag = status.MPI_TAG;
@@ -251,6 +375,71 @@ mpi_likelihood_worker (world_fmt * world, helper_fmt * helper, long rep)
     }
 }
 
+void
+mpi_startparam_master(world_fmt * world)
+{
+    long sender;
+    MPI_Status status;
+    double  *tmp;
+    int tag;
+    long i;
+    int numreceived = 0;
+    long workerloci=0;
+    MYMPIBARRIER(comm_world);
+    tmp = (double*) calloc(world->numpop2+1,sizeof(double));
+    while (numreceived < world->loci)
+      {
+//        printf("wait for data \n");
+        MYMPIRECV (tmp, world->numpop2+1, MPI_DOUBLE, MPI_ANY_SOURCE,
+                   MPI_ANY_TAG, comm_world, &status);
+        sender = status.MPI_SOURCE;
+        tag = status.MPI_TAG;
+ //       printf("received data form node %i with tag %i\n",sender,tag);
+        workerloci=tmp[0];
+        for(i=0; i<world->numpop2; i++)
+            world->param0[i] += tmp[i+1];
+        numreceived+=workerloci;
+      }
+    for(i=0; i<world->numpop2; i++)
+        world->param0[i] /= world->loci;
+ // 
+    free(tmp);
+}
+
+void
+mpi_startparam_worker (world_fmt * world)
+{
+    long ww;
+    long repstart;
+    long repstop;
+    long r;
+    long i;
+    long locus;
+    double *tmp;
+    MYMPIBARRIER(comm_world);
+    if(locidone>0)
+      {
+//        fprintf("send startparam from node %i\n",myID);
+    tmp = (double*) calloc(world->numpop2+1,sizeof(double));
+        set_replicates (world, world->repkind, world->options->replicatenum,
+                        &repstart, &repstop);
+        tmp[0]=(double)locidone;
+        for (ww = 0; ww < locidone; ww++)
+        {
+            locus = world->who[ww];
+            for (r = repstart; r < repstop; r++)
+            {
+                for(i=0; i < world->numpop2; i++)
+                    tmp[i+1] += world->atl[r][locus].param[i];
+            }
+        }
+        for(i=1; i < world->numpop2+1; i++)
+            tmp[i] /= locidone * (repstop-repstart);
+    MYMPISEND (tmp, world->numpop2+1, MPI_DOUBLE, MASTER, myID, comm_world);
+    free(tmp);
+}
+}
+
 
 void
 mpi_gmax_master (world_fmt * world, long *Gmax)
@@ -261,10 +450,10 @@ mpi_gmax_master (world_fmt * world, long *Gmax)
     int tag;
     int numreceived = 0;
     *Gmax = 0.;
-    MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (Gmax, 1, MPI_LONG, MASTER, comm_world);
     while (numreceived < numcpu - 1)
     {
-        MPI_Recv (&tmp, 1, MPI_LONG, MPI_ANY_SOURCE,
+        MYMPIRECV (&tmp, 1, MPI_LONG, MPI_ANY_SOURCE,
                   MPI_ANY_TAG, comm_world, &status);
         sender = status.MPI_SOURCE;
         tag = status.MPI_TAG;
@@ -272,7 +461,7 @@ mpi_gmax_master (world_fmt * world, long *Gmax)
             *Gmax = tmp;
         numreceived++;
     }
-    MPI_Bcast (Gmax, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (Gmax, 1, MPI_LONG, MASTER, comm_world);
 }
 
 void
@@ -284,7 +473,7 @@ mpi_gmax_worker (world_fmt * world)
     long r;
     long locus;
     long Gmax = 1;
-    MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (&Gmax, 1, MPI_LONG, MASTER, comm_world);
     set_replicates (world, world->repkind, world->options->replicatenum,
                     &repstart, &repstop);
 
@@ -297,7 +486,7 @@ mpi_gmax_worker (world_fmt * world)
                 Gmax = world->atl[r][locus].T;
         }
     }
-    MPI_Send (&Gmax, 1, MPI_LONG, MASTER, myID, comm_world);
+    MYMPISEND (&Gmax, 1, MPI_LONG, MASTER, myID, comm_world);
 }
 
 
@@ -312,7 +501,7 @@ mpi_send_stop (world_fmt * world)
     temp[0] = MIGMPI_END;
     for (worker = 1; worker < numcpu; worker++)
     {
-        MPI_Send (temp, (int) numelem2+2, MPI_DOUBLE, worker, 0, comm_world); //end of loci
+        MYMPISEND (temp, (int) numelem2+2, MPI_DOUBLE, worker, 0, comm_world); //end of loci
     }
     free (temp);
 }
@@ -325,7 +514,7 @@ mpi_send_stop_tag (int worker, world_fmt * world)
     long numelem2 = 2 * numelem;
     temp = (double *) calloc (numelem2+2, sizeof (double));
     temp[0] = MIGMPI_END;
-    MPI_Send (temp, (int) numelem2+2, MPI_DOUBLE, worker, 0, comm_world); //end of loci
+    MYMPISEND (temp, (int) numelem2+2, MPI_DOUBLE, worker, 0, comm_world); //end of loci
     free (temp);
 }
 
@@ -336,7 +525,7 @@ mpi_results_stop (void)
     long dummy = 0;
     for (worker = 1; worker < numcpu; worker++)
     {
-        MPI_Send (&dummy, 1, MPI_LONG, worker, 0, comm_world);
+        MYMPISEND (&dummy, 1, MPI_LONG, worker, 0, comm_world);
     }
 }
 
@@ -363,14 +552,14 @@ mpi_gradient_master (nr_fmt * nr, world_fmt * world, int *who)
     addon = (world->loci == 1)? 0 : 1;
     for (locus = 1; locus < MIN (world->loci + addon, numcpu); locus++)
     {
-        MPI_Send (temp, (int) numelem2+2, MPI_DOUBLE, locus, locus, comm_world);
-        MPI_Send (tempindex, (int) numelem, MPI_LONG, locus, locus, comm_world);
+        MYMPISEND (temp, (int) numelem2+2, MPI_DOUBLE, locus, locus, comm_world);
+        MYMPISEND (tempindex, (int) numelem, MPI_LONG, locus, locus, comm_world);
     }
     memset (nr->d, 0, sizeof (double) * numelem);
     for (locus = 1; locus < MIN (world->loci + addon, numcpu); locus++)
     {
         copy_and_clear_d (nr);
-        MPI_Recv (nr->d, (int) numelem, MPI_DOUBLE, MPI_ANY_SOURCE,
+        MYMPIRECV (nr->d, (int) numelem, MPI_DOUBLE, MPI_ANY_SOURCE,
                   MPI_ANY_TAG, comm_world, &status);
         add_back_d (nr);
         sender = status.MPI_SOURCE;
@@ -420,13 +609,13 @@ mpi_maximize_worker (world_fmt * world, long rep)
     set_replicates (world, world->repkind, rep, &repstart, &repstop);
 
     which_calc_like (world->repkind);
-    MPI_Bcast (&Gmax, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (&Gmax, 1, MPI_LONG, MASTER, comm_world);
     create_nr (nr, world, Gmax, 0, world->loci, world->repkind, repstart);
     SETUPPARAM0 (world, nr, world->repkind,
                  repstart, repstop, world->loci, MULTILOCUS, TRUE);
     while (!done)
     {
-        MPI_Recv (temp, (int) numelem2+2, MPI_DOUBLE, MASTER, MPI_ANY_TAG,
+        MYMPIRECV (temp, (int) numelem2+2, MPI_DOUBLE, MASTER, MPI_ANY_TAG,
                   comm_world, &status);
         locus = world->locus = status.MPI_TAG - 1;
         switch ((long) temp[0])
@@ -438,7 +627,7 @@ mpi_maximize_worker (world_fmt * world, long rep)
                     sizeof (double) * numelem);
             fill_helper (&helper, helper.expxv, helper.xv, world, nr);
             mpi_likelihood_worker (world, &helper, rep);
-            MPI_Send (nr->locilikes, (int) world->loci, MPI_DOUBLE, MASTER,
+            MYMPISEND (nr->locilikes, (int) world->loci, MPI_DOUBLE, MASTER,
                       locus + 1, comm_world);
             break;
         case MIGMPI_GRADIENT:
@@ -447,10 +636,10 @@ mpi_maximize_worker (world_fmt * world, long rep)
                     sizeof (double) * numelem);
             fill_helper (&helper, nr->param, nr->lparam, world, nr);
             nr->profilenum = temp[numelem2 + 1];
-            MPI_Recv (nr->indeks, (int) numelem, MPI_LONG, MASTER, MPI_ANY_TAG,
+            MYMPIRECV (nr->indeks, (int) numelem, MPI_LONG, MASTER, MPI_ANY_TAG,
                       comm_world, &status);
             mpi_gradient_worker (&helper, nr, world->atl);
-            MPI_Send (nr->d, (int) numelem, MPI_DOUBLE, MASTER, locus + 1,
+            MYMPISEND (nr->d, (int) numelem, MPI_DOUBLE, MASTER, locus + 1,
                       comm_world);
             break;
         case MIGMPI_RESULT:
@@ -482,9 +671,9 @@ broadcast_options_master (option_fmt * options)
     char *buffer;
     buffer = (char *) calloc (1, sizeof (char));
     bufsize = save_options_buffer (&buffer, options);
-    MPI_Barrier(comm_world);
-    MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
-    MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+    MYMPIBARRIER(comm_world);
+    MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
     free (buffer);
 }
 
@@ -494,11 +683,11 @@ broadcast_options_worker (option_fmt * options)
 {
     long bufsize;
     char *buffer, *sbuffer;
-    MPI_Barrier(comm_world);
-    MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBARRIER(comm_world);
+    MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
     buffer = (char *) calloc (bufsize + 1, sizeof (char));
     sbuffer = buffer;
-    MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+    MYMPIBCAST (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
     read_options_worker (&buffer, options);
     free (sbuffer);
 }
@@ -511,9 +700,9 @@ broadcast_data_master (data_fmt * data, option_fmt * options)
     char *buffer;
     buffer = (char *) calloc (1, sizeof (char));
     bufsize = pack_databuffer (&buffer, data, options);
-    MPI_Barrier(comm_world);
-    MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
-    MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+    MYMPIBARRIER(comm_world);
+    MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
     free (buffer);
 }
 
@@ -522,10 +711,10 @@ broadcast_data_worker (data_fmt * data, option_fmt * options)
 {
     long bufsize;
     char *buffer;
-    MPI_Barrier(comm_world);
-    MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBARRIER(comm_world);
+    MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
     buffer = (char *) calloc (bufsize, sizeof (char));
-    MPI_Bcast (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
+    MYMPIBCAST (buffer, bufsize, MPI_CHAR, MASTER, comm_world);
     unpack_databuffer (buffer, data, options);
     free (buffer);
 }
@@ -1145,7 +1334,7 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
     temp[1] = bufsize;
     for (locus = 1; locus < MIN (world->loci + 1, numcpu); locus++)
     {
-        MPI_Send (temp, numelem2, MPI_DOUBLE, locus, locus, comm_world);
+        MYMPISEND (temp, numelem2, MPI_DOUBLE, locus, locus, comm_world);
     }
     z = 0;
     while (!done)
@@ -1155,7 +1344,7 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
             break;
         }
 #ifdef BIGMEMORY
-        MPI_Recv (workload, 2*world->loci+2, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
+        MYMPIRECV (workload, 2*world->loci+2, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
                   comm_world, &status);
         bufsize = workload[2*world->loci] ;
         workerloci = workload[2*world->loci+1];
@@ -1163,7 +1352,7 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
         memset (buffer, 0, sizeof (char) * (bufsize + 1));
         sender = status.MPI_SOURCE;
         tag = status.MPI_TAG;
-        MPI_Recv (buffer, bufsize, MPI_CHAR, sender, tag, comm_world, &status);
+        MYMPIRECV (buffer, bufsize, MPI_CHAR, sender, tag, comm_world, &status);
         sbuffer = buffer;
         for(i=0; i<workerloci;i++)
         {
@@ -1175,13 +1364,13 @@ mpi_results_master (double sendtype, world_fmt * world, long maxreplicate,
         z += workerloci-1; //the test condition will add another one
 #else
 
-        MPI_Recv (&bufsize, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
+        MYMPIRECV (&bufsize, 1, MPI_LONG, MPI_ANY_SOURCE, MPI_ANY_TAG,
                   comm_world, &status);
         buffer = (char *) realloc (buffer, sizeof (char) * (bufsize + 1));
         memset (buffer, 0, sizeof (char) * (bufsize + 1));
         sender = status.MPI_SOURCE;
         tag = status.MPI_TAG;
-        MPI_Recv (buffer, bufsize, MPI_CHAR, sender, tag, comm_world, &status);
+        MYMPIRECV (buffer, bufsize, MPI_CHAR, sender, tag, comm_world, &status);
         sbuffer = buffer;
         (*unpack) (buffer, world, tag - 1, maxreplicate, numpop);
 #endif
@@ -1233,8 +1422,8 @@ mpi_results_worker (long bufs, world_fmt * world, long maxrep,
     }
     workload[2*world->loci]=strlen(allbuffer)+1;
     workload[2*world->loci+1]=(long)locidone;
-    MPI_Send (workload, 2+world->loci*2, MPI_LONG, MASTER, myID, comm_world);
-    MPI_Send (allbuffer, workload[2*world->loci], MPI_CHAR, MASTER, myID, comm_world);
+    MYMPISEND (workload, 2+world->loci*2, MPI_LONG, MASTER, myID, comm_world);
+    MYMPISEND (allbuffer, workload[2*world->loci], MPI_CHAR, MASTER, myID, comm_world);
     free(workload);
 #else
 
@@ -1244,8 +1433,8 @@ mpi_results_worker (long bufs, world_fmt * world, long maxrep,
         //    buf = &allbuffer;
         locus = world->who[ww];
         bufsize = (*pack) (&allbuffer, world, locus, maxrep, numpop);
-        MPI_Send (&bufsize, 1, MPI_LONG, MASTER, locus + 1, comm_world);
-        MPI_Send (allbuffer, bufsize, MPI_CHAR, MASTER, locus + 1, comm_world);
+        MYMPISEND (&bufsize, 1, MPI_LONG, MASTER, locus + 1, comm_world);
+        MYMPISEND (allbuffer, bufsize, MPI_CHAR, MASTER, locus + 1, comm_world);
     }
 
 #endif /*BIGMEMORY*/
@@ -1307,23 +1496,24 @@ mpi_broadcast_results (world_fmt * world, long loci,
             strcat(allbuffer,buffer);
             free(buffer);
         }
-        workload[2*loci]=strlen(allbuffer)+1;
+        workload[2*loci]=allbufsize;
         workload[2*loci+1]=(long)loci;
-        MPI_Barrier(comm_world);
-        MPI_Bcast (workload, 2 * (loci) + 2, MPI_LONG, MASTER, comm_world);
-        MPI_Bcast (allbuffer, allbufsize+1, MPI_CHAR, MASTER, comm_world);
+        MYMPIBARRIER(comm_world);
+        MYMPIBCAST (workload, 2 * (loci) + 2, MPI_LONG, MASTER, comm_world);
+        MYMPIBCAST (allbuffer, allbufsize+1, MPI_CHAR, MASTER, comm_world);
         //printf("%i> Locus %li results sent\n",myID, locus);
     }
     else
     { // worker node
 
-        MPI_Barrier(comm_world);
-        MPI_Bcast (workload, 2*(loci)+2, MPI_LONG, MASTER, comm_world);
+        MYMPIBARRIER(comm_world);
+        MYMPIBCAST (workload, 2*(loci)+2, MPI_LONG, MASTER, comm_world);
         bufsize =  workload[2*loci] ;
         workerloci = workload[2*loci+1];
-        allbuffer = (char *) realloc (allbuffer, sizeof (char) * (bufsize+ 1));
+        allbuffer = (char *) realloc (allbuffer,
+                                      sizeof (char) * (bufsize + 1));
         memset (allbuffer, 0, sizeof (char) * (bufsize + 1));
-        MPI_Bcast (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
+        MYMPIBCAST (allbuffer, bufsize + 1, MPI_CHAR, MASTER, comm_world);
         for(i=0; i<workerloci; i++)
         {
             locus = (int) workload[2*i];
@@ -1340,15 +1530,15 @@ mpi_broadcast_results (world_fmt * world, long loci,
         {
             bufsize =(*pack) (&allbuffer, world, locus, maxreplicate,
                               world->numpop);
-            MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
-            MPI_Bcast (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
+            MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+            MYMPIBCAST (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
             // printf("%i> Locus %li results sent\n",myID, locus);
         }
         else
         {
-            MPI_Bcast (&bufsize, 1, MPI_LONG, MASTER, comm_world);
+            MYMPIBCAST (&bufsize, 1, MPI_LONG, MASTER, comm_world);
             allbuffer = (char *) realloc (allbuffer, sizeof (char) * bufsize + 1);
-            MPI_Bcast (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
+            MYMPIBCAST (allbuffer, bufsize, MPI_CHAR, MASTER, comm_world);
             (*unpack)(allbuffer, world, locus, maxreplicate,
                       world->numpop);
             //printf("%i> Locus %li results received\n",myID, locus);
@@ -1383,7 +1573,7 @@ mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
 
     for (pnum = 0; pnum < MIN (nparam, numcpu - 1); pnum++)
     {
-        MPI_Send (&pnum, 1, MPI_LONG, pnum + 1, pnum + 1, comm_world);
+        MYMPISEND (&pnum, 1, MPI_LONG, pnum + 1, pnum + 1, comm_world);
         numsent++;
     }
     for (pnum = 0; pnum < nparam; pnum++)
@@ -1391,7 +1581,7 @@ mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
         done = FALSE;
         while(!done)
         {
-            MPI_Recv (tempstr, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG, comm_world, &status);
+            MYMPIRECV (tempstr, MAXBUFSIZE, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG, comm_world, &status);
             switch(tempstr[0])
             {
             case 'M':
@@ -1407,7 +1597,7 @@ mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
                 *buffer =
                     (char *) realloc (*buffer, sizeof (char) * (bufsize + quantsize + 1));
                 memset (*buffer, 0, sizeof (char) * (bufsize + quantsize + 1));
-                MPI_Recv (*buffer, bufsize + quantsize, MPI_CHAR, sender, MPI_ANY_TAG,
+                MYMPIRECV (*buffer, bufsize + quantsize, MPI_CHAR, sender, MPI_ANY_TAG,
                           comm_world, &status);
                 if(world->options->printprofsummary)
                 {
@@ -1425,14 +1615,21 @@ mpi_profiles_master (world_fmt * world, long nparam, int *profilewho)
         }
         if (numsent < nparam)
         {
-            MPI_Send (&numsent, 1, MPI_LONG, sender, numsent + 1, comm_world);
+            MYMPISEND (&numsent, 1, MPI_LONG, sender, numsent + 1, comm_world);
             numsent++;
         }
         else
         {
-            MPI_Send (&nparam, 1, MPI_LONG, sender, 0, comm_world); //end of parameter list
+            // stop worker because there is nothing to do anymore
+            MYMPISEND (&nparam, 1, MPI_LONG, sender, 0, comm_world); //end of parameter list
         }
     }
+    // stop workers that did nothing for profiles
+    for (sender = MIN (nparam, numcpu - 1) + 1; sender < numcpu ; sender++)
+    {
+        // stop all nodes to wait for profiles
+        MYMPISEND (&nparam, 1, MPI_LONG, sender, 0, comm_world); 
+    }
 }
 
 void
@@ -1450,7 +1647,7 @@ mpi_profiles_worker (world_fmt * world, long *gmaxptr)
     tempstr = (char *) calloc (MAXBUFSIZE, sizeof (char));
     while (!done)
     {
-        MPI_Recv (&pnum, 1, MPI_LONG, MASTER, MPI_ANY_TAG, comm_world, &status);
+        MYMPIRECV (&pnum, 1, MPI_LONG, MASTER, MPI_ANY_TAG, comm_world, &status);
         if (status.MPI_TAG != 0) //stop condition
         {
             // fills world->buffer with profile information
@@ -1467,8 +1664,8 @@ mpi_profiles_worker (world_fmt * world, long *gmaxptr)
             else
                 temp[2] = 0;
             tempstrsize = 1 + sprintf(tempstr,"P%li %li %li", temp[0],temp[1],temp[2]);
-            MPI_Send (tempstr,tempstrsize, MPI_CHAR, MASTER, pnum + 1, comm_world);
-            MPI_Send (world->buffer, temp[1] + temp[2], MPI_CHAR, MASTER,
+            MYMPISEND (tempstr,tempstrsize, MPI_CHAR, MASTER, pnum + 1, comm_world);
+            MYMPISEND (world->buffer, temp[1] + temp[2], MPI_CHAR, MASTER,
                       pnum + 1, comm_world);
             world->profilewho[profiledone++] = pnum;
         }
@@ -1550,13 +1747,13 @@ assignloci_worker (world_fmt * world)
     MPI_Status status;
     while (!done)
     {
-        MPI_Recv (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
+        MYMPIRECV (&locus, 1, MPI_LONG, MASTER, MPI_ANY_TAG,
                   comm_world, &status);
         if (status.MPI_TAG != 0) //stop condition
         {
             swap_atl (locus, locidone, world);
 
-            MPI_Send (&locus, 1, MPI_LONG, MASTER, locus + 1, comm_world);
+            MYMPISEND (&locus, 1, MPI_LONG, MASTER, locus + 1, comm_world);
             /* we want to know what locus we worked for
                - to control the work sent by master
                - to use in setup_parameter0() [combroyden2.c] */
@@ -1725,7 +1922,7 @@ void setup_filehandle_db(FILE *file, world_fmt *world, option_fmt *options, data
     long filehandle = get_filehandle(file, world, options, data);
     filedb[filenum].file = file;
     filedb[filenum++].handle = filehandle;
-    fprintf(stdout,"filedb: %p %li\n",file,filehandle);
+  //  fprintf(stdout,"filedb %li: %p %li\n",filenum, file,filehandle);
 }
 
 long retrieve_filehandle(FILE *file)
@@ -1776,6 +1973,7 @@ long get_filehandle2(FILE *file, world_fmt *world)
     if(file == world->bayesfile)
         return BAYESFILENUM;
 #endif
+ //   fprintf(stdout,"@@@@@@@@@@@@@@@wrong wrong wrong@@@@@@@@@@@@@@@@\n");
     return STDOUTNUM;
 }
 
@@ -1831,7 +2029,7 @@ mpi_fprintf(FILE *file, const char *fmt, ...)
     if(myID!=MASTER)
     {
         bufsize= 1+strlen(p);
-        MPI_Send (p, bufsize, MPI_CHAR, MASTER, myID, comm_world);
+        MYMPISEND (p, bufsize, MPI_CHAR, MASTER, myID, comm_world);
     }
     else
         fprintf(file,"%s", p);
