@@ -18,14 +18,14 @@
  
 Copyright 2001 Peter Beerli and Joseph Felsenstein
  
-$Id: options.h,v 1.19 2002/06/20 06:25:37 beerli Exp $
+$Id: options.h,v 1.20 2002/10/23 22:43:43 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
 
 extern void create_options (option_fmt ** options);
 extern void init_options (option_fmt * options);
-extern void get_options (option_fmt * options);
+//extern void get_options (option_fmt * options);
 extern void set_param (world_fmt * world, data_fmt * data,
                            option_fmt * options, long locus);
 extern void set_profile_options (option_fmt * options);
@@ -48,4 +48,6 @@ extern void resynchronize_param (world_fmt * world);
 extern long save_options_buffer (char **buffer, option_fmt * options);
 extern void read_options_master (option_fmt * options);
 extern void read_options_worker (char **buffer, option_fmt * options);
+extern long scan_connect (char *custm2, long start, long stop, char check);
+
 #endif /*OPTIONS_INCLUDE */
