@@ -15,7 +15,7 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: profile.c,v 1.62 2002/08/12 21:13:55 beerli Exp $
+$Id: profile.c,v 1.63 2002/11/09 20:06:13 beerli Exp $
  
 -------------------------------------------------------*/
 
@@ -430,7 +430,7 @@ void
 print_profile_title (world_fmt * world)
 {
     print_line (world->outfile, '=', 79, CONT);
-    FPRINTF (world->outfile, "Profile likelihood tables\n");
+    FPRINTF (world->outfile, "Profile likelihood tables   [Summary is at the end of the file]\n");
     print_line (world->outfile, '=', 79, CONT);
 }
 
