@@ -21,7 +21,7 @@ Copyright 2001 Peter Beerli and Joseph Felsenstein
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: random.c,v 1.30 2002/06/20 06:25:37 beerli Exp $
+$Id: random.c,v 1.33 2002/12/22 04:50:45 beerli Exp $
 -------------------------------------------------------*/
 
 #include "sighandler.h"
@@ -29,18 +29,25 @@ $Id: random.c,v 1.30 2002/06/20 06:25:37 beerli Exp $
 #include "heating.h"
 #include "random.h"
 #include "migrate_mpi.h"
+
 #ifdef SPRNG   /*NCSA random number generator library, slower than ours */
-#define SIMPLE_SPRNG
-#ifdef MPI
-#ifndef USE_MPI
-#define USE_MPI
+    #define SIMPLE_SPRNG
+    #ifdef MPI
+        #ifndef USE_MPI
+            #define USE_MPI
+        #endif
+    #endif
+    #include "sprng.h"
 #endif
+
+#ifdef MERSENNE_TWISTER
+#include "mtwist-0.4/mtwist.h"
 #endif
-#include "sprng.h"
-#endif
+
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
+
 #ifdef PTHREADS
 extern tpool_t heating_pool;
 #endif
@@ -59,19 +66,19 @@ void
 getseed (option_fmt * options)
 {
 #ifdef MPI
-    long test;
+    long test=0;
 #endif
 #ifndef SPRNG
-
+#ifndef MERSENNE_TWISTER
     long i;
 #endif
-
+#endif
     long timeseed;
-
     switch (options->autoseed)
     {
     case AUTO:
         timeseed = abs ((long) time (NULL));
+#ifndef MERSENNE_TWISTER
         switch (timeseed % 4)
         {
         case 0:
@@ -88,6 +95,7 @@ getseed (option_fmt * options)
             error ("Problem with automatic random number seed assignment");
         }
         options->inseed = abs ((long) timeseed);
+#endif /*Mersenne-Twister*/
         break;
     case NOAUTO:
         break;
@@ -99,9 +107,10 @@ getseed (option_fmt * options)
     }
 #ifndef SPRNG
 #ifdef MPI
-    MPI_Bcast (&options->inseed, 1, MPI_LONG, MASTER, comm_world);
+    MYMPIBCAST (&options->inseed, 1, MPI_LONG, MASTER, comm_world);
     if (myID != MASTER)
     {
+#ifndef MERSENNE_TWISTER    
         for (i = 1; i <= 1000; i++) /* clear the random number generator */
             RANDUM ();
 
@@ -115,32 +124,39 @@ getseed (option_fmt * options)
             i++;
         }
         while (options->inseed != 0);
-
-        for (i = 0; i < myID; i++)
-        {
+        for (i = 0; i <= myID; i++)
+          {
             test = RANDINT (1, MAXLONG);
             //printf("     %i %li> %li\n", myID, i, test);
-        }
+          }
         switch (test % 4)
-        {
-        case 0:
-            test++;
-            break;
-        case 1:
-            break;
-        case 2:
-            test--;
-            break;
-        case 3:
-            test--;
-            test--;
-            break;
-        }
+          {
+            case 0:
+                test++;
+                break;
+            case 1:
+                break;
+            case 2:
+                test--;
+                break;
+            case 3:
+                test--;
+                test--;
+                break;
+          }
         options->inseed = test;
     }
-    printf ("%i> random number seed = %li\n", myID, options->inseed);
-#endif
-#endif
+    FPRINTF (stdout, "%i> random number seed = %li\n", myID, options->inseed);
+#else
+        mt_seed32new((unsigned long)options->inseed);
+       // for (i = 0; i <= myID; i++)
+       //   {
+       //     test = RANDINT (1, MAXLONG);
+            //printf("     %i %li> %li\n", myID, i, test);
+       //   }        
+#endif /*MERSENNE_TWISTER*/
+#endif /*MPI*/
+#endif /*SPRNG*/
 
     options->saveseed = options->inseed;
 #ifdef SPRNG
@@ -150,7 +166,7 @@ getseed (option_fmt * options)
 
     //    for (i = 1; i <= 1000; i++) /* clear the random number generator */
     //        RANDUM ();
-
+#ifndef MERSENNE_TWISTER
     for (i = 0; i <= 2; i++)
         seed[i] = 0;
     i = 0;
@@ -161,6 +177,9 @@ getseed (option_fmt * options)
         i++;
     }
     while (options->inseed != 0);
+#else
+    mt_seed32new((unsigned long) options->inseed);
+#endif /*MERSENNE_TWISTER*/
 #endif /* not SPRNG */
 }
 
@@ -220,7 +239,9 @@ randum (void)
 #ifdef SPRNG
     return sprng ();
 #else
-
+#ifdef MERSENNE_TWISTER
+    return mt_drand();
+#else
 double value;
 //  longer newseed;
 newseed[0] = 1549 * seed[0];
@@ -237,6 +258,7 @@ swap_ptr (&newseed, &seed);
 seed[2] &= 1023;
 value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
 return value;
+#endif /*MERSENNE_TWISTER*/
 #endif /*SPRNG*/
 }    /* randum */
 #endif
